/*
  Filename:	main.c
  Author:	Ron Burkey
  Legal:	I, Ron Burkey, am placing this file in the public domain.
  Purpose:     	A simple regular-expression sample program for the IMCROSS project.
  Mod history:	2008-04-08 RSB	Created.

  This sample program is a kind of super-simplified grep.  You specify a 
  pattern (which is an "extended" regular expression) and a filename on the
  command-line, and you get back a print of all lines containing the pattern. 
  The match is case-sensitive.  The exit code is the number of matching lines.
*/

#include <stdio.h>
#include <string.h>
#include <regex.h>

// A buffer for strings.
static char s[1024];

int
main (int argc, char *argv[])
{
  int i, Count = 0;
  char *Pattern = NULL, *File = NULL;
  regex_t CompiledPattern;
  regmatch_t pmatch[1];
  FILE *fp = NULL;
  
  // Parse the command-line arguments.
  for (i = 1; i < argc; i++)
    {
      if (!strncmp (argv[i], "--pattern=", 10))
        Pattern = &argv[i][10];
      else if (!strncmp (argv[i], "--file=", 7))
        File = &argv[i][7];
      else
        {
	Help:
	  fprintf (stderr, "USAGE:\n");
	  fprintf (stderr, "\tsample_RegularExpressions \"--pattern=P\" \"--file=F\"\n");
	  fprintf (stderr, "The pattern P is an extended regular expression.  The\n");
	  fprintf (stderr, "quotation marks are present in case the pattern or \n");
	  fprintf (stderr, "filename contains spaces or funky characters.\n");
	  goto Error;
	}
    }
  if (Pattern == NULL || File == NULL)
    goto Help;
    
  // Open the specified input file.
  fp = fopen (File, "r");
  if (fp == NULL)
    {
      fprintf (stderr, "File \"%s\" does not exist.\n", File);
      goto Error;
    }
    
  // "Compile" the regular expression.
  i = regcomp (&CompiledPattern, Pattern, REG_NEWLINE | REG_EXTENDED);
  if (i != 0)
    {
      regerror (i, &CompiledPattern, s, sizeof (s));
      fprintf (stderr, "%s\n", s);
      goto Error;
    }

  // Read the input file line by line and find matches.
  while (NULL != fgets (s, sizeof (s), fp))
    {
      i = regexec (&CompiledPattern, s, 1, pmatch, 0);
      if (i == 0)
        {
          printf ("%s", s);
	  Count++;
	}
    }
  
Error:
  if (fp != NULL)
    fclose (fp);
  return (Count);
}

